package transport

import (
	// "fmt"
	"net/http"

	"github.com/labstack/echo/v4"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_seminar/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_seminar/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_seminar/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_seminar/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type AdminBatchSeminarHandler struct {
	batchSeminarUsecase usecase.BatchSeminarUsecase
}

func NewAdminBatchSeminarHandler(cfg config.BatchSeminarTransportConfig) *AdminBatchSeminarHandler {
	batchSeminarRepository := repository.NewBatchSeminarRepository(cfg.DBWrite, cfg.DBRead)
	batchSeminarUsecase := usecase.NewBatchSeminarUseCase(batchSeminarRepository)

	return &AdminBatchSeminarHandler{
		batchSeminarUsecase: batchSeminarUsecase,
	}

}

func (t *AdminBatchSeminarHandler) MountAdmin(group *echo.Group) {
	group.POST("/batch-seminar", t.AddBatchSeminar)
	group.GET("/batch-seminar", t.GetBatchSeminar)
	group.DELETE("/batch-seminar", t.DeleteBatchSeminar)
	group.GET("/batch-seminar-current", t.GetBatchSeminarCurrent)
	group.PATCH("/batch-seminar-live", t.SetLive)
	group.GET("/batch-seminar-live", t.GetLive)
}

// AddBatchSeminar adds a new batch for Seminar.
// @Summary Add batch for Seminar
// @Description Adds a new batch for Seminar.
// @Tags Batch Seminar
// @Accept json
// @Produce json
// @Param batch body entity.BatchSeminar true "Batch Seminar object"
// @Success 200 {object} entity.BatchSeminar "Batch added successfully"
// @Router /batch-seminar [post]
func (t *AdminBatchSeminarHandler) AddBatchSeminar(c echo.Context) error {
	request := new(entity.BatchSeminar)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	target := &entity.BatchSeminar{
		StartDate: request.StartDate,
		EndDate:   request.EndDate,
	}
	newBatch, err := t.batchSeminarUsecase.AddBatchSeminar(*target)
	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusCreated, "Batch added successfully", newBatch))
}

// GetBatchSeminar retrieves a list of batch for Seminar.
// @Summary Get batch for TA
// @Description Retrieves a list of batch for Seminar.
// @Tags Batch Seminar
// @Accept json
// @Produce json
// @Success 200 {object} []entity.BatchSeminar "Batch seminar successfully retreived"
// @Router /batch-seminar [get]
func (t *AdminBatchSeminarHandler) GetBatchSeminar(c echo.Context) error {
	batch, err := t.batchSeminarUsecase.GetBatchSeminar()
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Batch seminar successfully retreived", batch))
}

// DeleteBatchSeminar deletes a batch for Seminar.
// @Summary Delete batch for Seminar
// @Description Deletes a batch for Seminar.
// @Tags Batch Seminar
// @Accept json
// @Produce json
// @Param batch body entity.BatchSeminar true "Batch Seminar object"
// @Success 200 {object} entity.BatchSeminar  "Batch Seminar successfully deleted"
// @Router /batch-seminar [delete]
func (t *AdminBatchSeminarHandler) DeleteBatchSeminar(c echo.Context) error {
	request := new(entity.BatchSeminar)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}
	batch, err := t.batchSeminarUsecase.DeleteBatchSeminar(request.IdBatch)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Batch Seminar successfully deleted", batch))
}

// GetBatchSeminarCurrent retrieves a list of batch for Seminar that current time is between its startDate and endDate.
// @Summary Get batch seminar that current time is between its startDate and endDate.
// @Description Retrieves a list of batch for Seminar.
// @Tags Batch Seminar
// @Accept json
// @Produce json
// @Success 200 {object} []entity.BatchSeminar "Batch seminar successfully retreived"
// @Router /batch-seminar-current [get]
func (t *AdminBatchSeminarHandler) GetBatchSeminarCurrent(c echo.Context) error {
	batch, err := t.batchSeminarUsecase.GetBatchSeminarCurrent()
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Batch seminar successfully retreived", batch))
}

// SetLive sets a batch for seminar to be live.
// @Summary Set Batch to live
// @Description Set Batch to live.
// @Tags Batch Seminar
// @Accept json
// @Produce json
// @Param batch body entity.BatchSeminar true "Batch Seminar object"
// @Success 200 {object} entity.BatchSeminar  "Batch Seminar successfully updated to live"
// @Router /batch-seminar-live [patch]
func (t *AdminBatchSeminarHandler) SetLive(c echo.Context) error {
	request := new(entity.BatchSeminar)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	batch, err := t.batchSeminarUsecase.SetLiveBatch(request.IdBatch)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Batch Seminar successfully updated to live", batch))
}

// GetLive retrieves a batch for Seminar that its status is live.
// @Summary Get batch seminar that is live.
// @Description Retrieves a batch for Seminar.
// @Tags Batch Seminar
// @Accept json
// @Produce json
// @Success 200 {object} []entity.BatchSeminar "Batch seminar successfully retreived"
// @Router /batch-seminar-live [get]
func (t *AdminBatchSeminarHandler) GetLive(c echo.Context) error {
	batch, err := t.batchSeminarUsecase.GetBatchSeminarLive()
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Batch seminar successfully retreived", batch))
}
