package transport

import (
	"net/http"

	"github.com/labstack/echo/v4"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_tim_tugas/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_tim_tugas/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_tim_tugas/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_tim_tugas/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

// DashboardTimTugasHandler handles HTTP requests related to dashboard statistics for tim tugas.
type AdminDashboardTimTugasHandler struct {
	dashboardTimTugasUseCase usecase.DashboardTimTugasUc
}

// NewDashboardTimTugasHandler creates a new instance of NewDashboardTimTugasHandler.
func NewDashboardTimTugasHandler(cfg config.DashboardTimTugasTransportConfig) *AdminDashboardTimTugasHandler {
	dashboardTimTugasRepository := repository.NewDashboardTimTugasRepository(cfg.DBRead)
	dashboardTimTugasUseCase := usecase.NewDashboardTimTugasUc(dashboardTimTugasRepository)

	return &AdminDashboardTimTugasHandler {
		dashboardTimTugasUseCase: *dashboardTimTugasUseCase,
	}
}

// MountAdmin mounts the admin routes for dashboard tim tugas management.
func (admin *AdminDashboardTimTugasHandler) MountAdmin(group *echo.Group) {
	group.GET("/statistics-dashboard-tim-tugas", admin.GetDashboardStatistics)
	group.GET("/period-data", admin.GetPeriodData)
	// group.GET("/stats-mahasiswa", admin.GetStatsDaftarMahasiswa)
}

// GetDashboardStatistics retrieves the tim tugas dashboard data and statistics.
// @Summary Retrieve dashboard data and statistics
// @Description Retrieve data and statistics that is used for tim tugas dashboard
// @Tags dashboard-tim-tugas
// @Produce json
// @Success 200 {object} entity.DashboardData
// @Router /statistics-dashboard-tim-tugas [get]
func (admin *AdminDashboardTimTugasHandler) GetDashboardStatistics(c echo.Context) error {
	stats, err := admin.dashboardTimTugasUseCase.GetDashboardStatistics()
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	dataPendaftar, err := admin.dashboardTimTugasUseCase.GetDaftarMahasiswa()
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	reponseData := entity.DashboardData {
		ListPendaftar: 	dataPendaftar,
		Stats: 			stats,
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Data and Statistics retrieved", reponseData))
}

// GetPeriodData retrieves the tim tugas dashboard period statistics (period data and number of mahasiswa accepted).
// @Summary Retrieve dashboard period statistics (period data and number of mahasiswa accepted)
// @Description Retrieve period statistics (period data and number of mahasiswa accepted) that is used for tim tugas dashboard
// @Tags dashboard-tim-tugas
// @Produce json
// @Success 200 {object} entity.PeriodStats
// @Router /period-data [get]
func (admin *AdminDashboardTimTugasHandler) GetPeriodData(c echo.Context) error {
	periodStats, err := admin.dashboardTimTugasUseCase.GetPeriodData()
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, "Failed to get period data", nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Period data retrieved", periodStats))
}

// func (admin *AdminDashboardTimTugasHandler) GetStatsDaftarMahasiswa(c echo.Context) error {

// }